var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { getItemResource as getResource, getItemResources as getResources, } from '@esri/arcgis-rest-portal';
import { createOperation, parallel, series, } from '../../../utilities/index.js';
import { ItemResourceOperationTypes, } from './types.js';
/**
 * Fetches an item resource and optionally parses it to the correct format.
 *
 * @param requestOptions - Options for the request
 * @param requestOptions.id - Item Id
 * @param resource - Resource info
 * @returns A PProgress Promise that will resolve with the resource
 */
export function getItemResource(_a, resource) {
    var id = _a.id, requestOptions = __rest(_a, ["id"]);
    return createOperation(ItemResourceOperationTypes.GET_ITEM_RESOURCE, [getResource(id, __assign(__assign({}, requestOptions), resource))], __assign({ id: id }, requestOptions));
}
/**
 * Get the resources associated with an item.
 *
 * @param requestOptions - Options for the request
 * @param requestOptions.id - Item Id
 * @returns A PProgress Promise that will resolve with the resources associated with an item
 */
export function getItemResources(_a) {
    var id = _a.id, requestOptions = __rest(_a, ["id"]);
    var getRemainingPages = function (startPage) {
        // build array of requests for additional pages
        var pageRequests = [];
        for (var start = startPage.nextStart; start <= startPage.total; start += startPage.num) {
            var pageOptions = __assign(__assign({}, requestOptions), { params: __assign(__assign({}, requestOptions.params), { start: start, num: startPage.num }) });
            pageRequests.push(getResources(id, pageOptions));
        }
        // combine page responses into complete response
        return parallel.apply(void 0, pageRequests).then(function (pageResponses) {
            return pageResponses.reduce(function (combined, pageResponse) { return (__assign(__assign({}, combined), { nextStart: -1, num: combined.num + pageResponse.num, resources: __spreadArray(__spreadArray([], combined.resources, true), pageResponse.resources, true) })); }, startPage);
        });
    };
    return createOperation(ItemResourceOperationTypes.GET_ITEM_RESOURCES, [
        series(
        // get requested page of resources
        getResources(id, requestOptions), 
        // get additional pages if needed and no paging parameters were passed
        function (response) {
            var _a, _b;
            if (response.num < response.total &&
                response.nextStart > 0 &&
                ((_a = requestOptions.params) === null || _a === void 0 ? void 0 : _a.start) === undefined &&
                ((_b = requestOptions.params) === null || _b === void 0 ? void 0 : _b.num) === undefined) {
                return getRemainingPages(response);
            }
            return response;
        }),
    ], __assign({ id: id }, requestOptions));
}
//# sourceMappingURL=get.js.map