var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { addFeatures as addFeaturesToService, } from '@esri/arcgis-rest-feature-layer';
import { addToServiceDefinition, createFeatureService as createHostedFeatureService, } from '@esri/arcgis-rest-service-admin';
import { createOperation } from '../../../utilities/index.js';
import { FeatureServiceOperationTypes, } from './types.js';
/**
 * Add Features to a feature layer.
 *
 * @param requestOptions - Options for the request
 * @param url - Layer service url
 * @param features - Array of JSON features to add
 * @returns A PProgress Promise that will resolve with the addFeatures response.
 */
export function addFeatures(requestOptions, url, features) {
    return createOperation(FeatureServiceOperationTypes.ADD_FEATURES, [
        addFeaturesToService(__assign(__assign({}, requestOptions), { url: url, features: features })),
    ]);
}
/**
 * Add layer(s) and/or table(s) to a hosted feature service.
 *
 * @param requestOptions - Options for the request
 * @param url - URL of feature service
 * @param definition - Service definition
 * @param definition.layers - Layers to add
 * @param definition.tables - Tables to add
 * @returns A PProgress Promise that resolves with service layer and/or table details once the definition has been updated.
 */
export function addToFeatureServiceDefinition(requestOptions, url, _a) {
    var layers = _a.layers, tables = _a.tables;
    return createOperation(FeatureServiceOperationTypes.ADD_FEATURE_SERVICE_DEFINITION, [addToServiceDefinition(url, __assign(__assign({}, requestOptions), { layers: layers, tables: tables }))]);
}
/**
 * Create a new hosted feature service.
 *
 * @param requestOptions - Options for the request
 * @param service - A JSON object specifying the properties of the newly-created service.
 * @returns A PProgress Promise that resolves with service details once the service has been created.
 */
export function createFeatureService(requestOptions, service) {
    return createOperation(FeatureServiceOperationTypes.CREATE_FEATURE_SERVICE, [
        createHostedFeatureService(__assign(__assign({}, requestOptions), { item: service })),
    ]);
}
//# sourceMappingURL=add.js.map