var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { getPortalUrl } from '@esri/arcgis-rest-portal';
import PProgress from 'p-progress';
import pWaterfall from 'p-waterfall';
import { v4 as uuidv4 } from 'uuid';
import Debugger from './debug.js';
/**
 * Creates an operation from a list of steps to execute.
 * @param type Operation type
 * @param steps Steps to execute to complete the operation
 * @param source Source item reference
 * @param [destination] Destination item reference
 * @returns Promise w/operation info that will resolve with the data from the final step
 */
export function createOperation(type, steps, source, destination) {
    // initialize operation object
    var info = __assign(__assign(__assign({ type: type, id: uuidv4() }, (source && {
        source: {
            id: source.id,
            portalUrl: getPortalUrl(source),
        },
    })), (destination && {
        destination: {
            id: destination.id,
            portalUrl: getPortalUrl(destination),
        },
    })), { startTime: new Date() });
    // log operation info
    var debug = Debugger('operation')
        .extend(info.type)
        .extend(info.id.substring(0, 8));
    debug('operation initialized: %O', info);
    // add step completion/failure handlers
    steps.forEach(function (step, index) {
        Promise.resolve(step)
            .then(function (result) {
            return debug("step ".concat(index + 1, " of ").concat(steps.length, " completed: %o"), result);
        })
            .catch(function (err) {
            return debug("step ".concat(index + 1, " of ").concat(steps.length, " failed: "), err);
        });
    });
    // add promise to operation
    var operation = PProgress.all(steps)
        .onProgress(function (value) {
        return debug('operation progress: %d%%', value * 100);
    })
        .then(function (results) {
        // set operation end time
        info.endTime = new Date();
        // return last result
        var result = results.length ? results[results.length - 1] : undefined;
        debug('operation completed: %o', result);
        return result;
    }, function (err) {
        debug('operation failed: ', err);
        throw err;
    });
    // add operation info to promise (circular reference)
    info.promise = operation;
    operation.info = info;
    return operation;
}
/**
 * Waits for all operations to complete and returns an array of the results.
 * @param operations - Promise or expressions to wait for in parallel.
 * @returns Promise that will resolve with the data from all the operations
 */
export function parallel() {
    var operations = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        operations[_i] = arguments[_i];
    }
    return PProgress.all(operations);
}
/**
 * Executes functions in series, passing the result of each function to the next
 * @param operation Promise or expression to complete before executing additional functions
 * @param functions Functions to execute in series after the operation completes
 */
export function series(operation) {
    var _this = this;
    var functions = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        functions[_i - 1] = arguments[_i];
    }
    // return a progress promise
    return new PProgress(function (resolve, reject, progress) {
        // wrap functions with progress reporting function
        var tasks = __spreadArray([function () { return __awaiter(_this, void 0, void 0, function () { return __generator(this, function (_a) {
                return [2 /*return*/, operation];
            }); }); }], functions, true).map(function (originalFunction, index, array) {
            return function (previousValue) {
                var promise = PProgress.all([originalFunction(previousValue)])
                    // add listener to report total series progress on function progress
                    .onProgress(function (progressValue) {
                    return progress((index + progressValue) / array.length);
                })
                    // convert array result back to single value
                    .then(function (_a) {
                    var result = _a[0];
                    return result;
                });
                // report function progress immediately
                progress((index + promise.progress) / array.length);
                return promise;
            };
        });
        // execute tasks in series
        pWaterfall(tasks).then(resolve, reject);
    });
}
//# sourceMappingURL=operation.js.map