/**
 * File constructor for Microsoft Legacy Edge.
 *
 * @param fileBits Contents for file
 * @param fileName Name for file
 * @param options Bucket of options, such as `type` for the MIME type;
 * @return Blob mimicking a File
 * @see https://developer.mozilla.org/en-US/docs/Web/API/File/File
 * @see https://developer.mozilla.org/en-US/docs/Web/API/Blob/Blob
 */
function newFileMicrosftEdgeLegacy(fileBits, fileName, options) {
    var newOptions = options;
    if (typeof newOptions === 'undefined') {
        newOptions = { type: '' };
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    var blob = new Blob(fileBits, newOptions);
    blob.lastModified = new Date();
    blob.name = fileName;
    return blob;
}
/**
 * File constructor for NodeJS and form-data.
 *
 * @param fileBits Contents for file
 * @param fileName Name for file
 * @return A File stream.
 * @see https://www.npmjs.com/package/form-data
 */
function newFileNodeJS(fileBits, fileName) {
    var file = fileBits[0].stream();
    file.lastModified = new Date();
    file.name = fileName;
    file.path = fileName;
    return file;
}
/**
 * Supplies the File constructor for Microsoft Legacy Edge and NodeJS.
 *
 * @param fileBits Contents for file
 * @param fileName Name for file
 * @param options Bucket of options, such as `type` for the MIME type;
 * @return File for modern browsers, Blob for Microsoft Legacy Edge, or a File stream for NodeJS.
 * @see https://developer.mozilla.org/en-US/docs/Web/API/File/File
 * @see https://developer.mozilla.org/en-US/docs/Web/API/Blob/Blob
 */
// eslint-disable-next-line import/prefer-default-export
export function newFile(fileBits, fileName, options) {
    var file;
    // Modern browser
    try {
        file = new File(fileBits, fileName, options);
    }
    catch (_a) {
        // Microsoft Legacy Edge
        try {
            file = newFileMicrosftEdgeLegacy(fileBits, fileName, options);
        }
        catch (_b) {
            // NodeJS
            file = newFileNodeJS(fileBits, fileName);
        }
    }
    return file;
}
//# sourceMappingURL=ponyfills.js.map