var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { getItem, } from '../../core/index.js';
import { ItemTypeCategories } from '../../core/item/type/types.js';
import { createOperation, series } from '../../utilities/index.js';
import copyBaseItem from './base/index.js';
import copyDesktopItem from './desktop/index.js';
import { CombinedItemWorkflowOperationTypes, } from './types.js';
import copyWebItem from './web/index.js';
/**
 * Copies an item.
 *
 * @param requestOptions - Options for the source item request
 * @param destinationOptions - Options for the destination request
 * @param options - Copy options
 * @returns A PProgress Promise that will resolve with the status of the copy.
 */
export default function copyItem(requestOptions, destinationOptions, options) {
    var defaultOptions = { data: true, resources: true };
    var steps = [
        series(getItem(requestOptions, { data: false, resources: false }), function (item) {
            var copyOptions = __assign(__assign(__assign({}, defaultOptions), options), { item: item });
            switch (item.type.category) {
                case ItemTypeCategories.DESKTOP:
                    return copyDesktopItem(requestOptions, destinationOptions, copyOptions);
                case ItemTypeCategories.WEB:
                    return copyWebItem(requestOptions, destinationOptions, copyOptions);
                default:
                    return copyBaseItem(requestOptions, destinationOptions, copyOptions);
            }
        }),
    ];
    return createOperation(CombinedItemWorkflowOperationTypes.COPY_ITEM, steps, requestOptions);
}
//# sourceMappingURL=copy.js.map