import { ItemTypeClassifications } from '../../../core/item/type/types.js';
import { createOperation } from '../../../utilities/index.js';
import copyBaseItem from '../base/index.js';
import copyWebApplicationItem from './applications/index.js';
import copyWebLayerItem from './layer/index.js';
import { WebCategoryItemWorkflowOperationTypes, } from './types.js';
/**
 * Copy a web item, optionally including data and resources.
 *
 * @param requestOptions - Options for the source item request
 * @param destinationOptions - Options for the destination request
 * @param options - Copy options
 * @returns A PProgress Promise that will resolve with the status of the copy.
 */
export default function copyWebItem(requestOptions, destinationOptions, options) {
    var steps = [
        (function () {
            switch (options.item.type.classification) {
                case ItemTypeClassifications.APPLICATIONS:
                    return copyWebApplicationItem(requestOptions, destinationOptions, options);
                case ItemTypeClassifications.LAYERS:
                    return copyWebLayerItem(requestOptions, destinationOptions, options);
                default:
                    return copyBaseItem(requestOptions, destinationOptions, options);
            }
        })(),
    ];
    return createOperation(WebCategoryItemWorkflowOperationTypes.COPY_WEB_ITEM, steps, requestOptions);
}
//# sourceMappingURL=copy.js.map