var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { addFeatures, addItemData, addItemResource, addToFeatureServiceDefinition, createFeatureService, getFeatures, getFeatureServiceDefinition, getLayersAndTables, ItemTypes, updateItemDescription, } from '../../../../core/index.js';
import { createOperation, parallel, series, } from '../../../../utilities/index.js';
import { getItemContent } from '../../base/copy.js';
import copyBaseItem from '../../base/index.js';
import { WebLayerItemWorkflowOperationTypes, } from './types.js';
/**
 * Copy a feature service item.
 *
 * @param requestOptions - Options for the source item request
 * @param destinationOptions - Options for the destination request
 * @param options - Copy options
 * @returns A PProgress Promise that will resolve with the status of the copy.
 */
export function copyFeatureServiceItem(requestOptions, destinationOptions, options) {
    // function to retrieve service definitions
    var getSourceServiceDefinition = function (url) {
        return parallel(
        // get service definition
        getFeatureServiceDefinition(requestOptions, url), 
        // get layer/table definitions
        getLayersAndTables(requestOptions, url)).then(function (_a) {
            var service = _a[0], _b = _a[1], layers = _b.layers, tables = _b.tables;
            return (__assign(__assign({}, service), { layers: layers, tables: tables }));
        });
    };
    // function to create service from definition
    var createDestinationService = function (name, _a) {
        var layers = _a.layers, tables = _a.tables, definition = __rest(_a, ["layers", "tables"]);
        return series(createFeatureService(destinationOptions, __assign(__assign({}, definition), { name: name })).then(function (result) { return ({
            description: {
                id: result.itemId,
                type: result.type,
                title: result.name,
                url: result.serviceurl,
            },
        }); }), function (item) {
            return addToFeatureServiceDefinition(destinationOptions, item.description.url, __assign(__assign({}, definition), { layers: layers, tables: tables })).then(function (_a) {
                var success = _a.success, service = __rest(_a, ["success"]);
                return ({
                    item: item,
                    service: service,
                });
            });
        });
    };
    // function to map layer ids between source / destination services
    var handleLayerIdChange = function (source, destination) {
        var _a;
        // build source layer id -> destination layer id lookup
        // assumes that layer order is maintained
        var idLookup = __spreadArray(__spreadArray([], source.service.layers, true), source.service.tables, true).reduce(function (map, layer, index) {
            var _a;
            return (__assign(__assign({}, map), (layer.id !== undefined && (_a = {}, _a[layer.id] = index, _a))));
        }, {});
        // substitute layer ids in item data as function side effect
        if ((_a = destination.item.data) === null || _a === void 0 ? void 0 : _a.text) {
            try {
                var data_1 = JSON.parse(destination.item.data.text);
                ['layers', 'tables'].forEach(function (propertyName) {
                    var _a;
                    (_a = data_1[propertyName]) === null || _a === void 0 ? void 0 : _a.forEach(function (_a, index) {
                        var _b;
                        var id = _a.id;
                        data_1[propertyName][index].id = (_b = idLookup[id]) !== null && _b !== void 0 ? _b : id;
                    });
                });
                // eslint-disable-next-line no-param-reassign
                destination.item.data.text = JSON.stringify(data_1);
            }
            catch (_b) {
                // no-op
            }
        }
        return idLookup;
    };
    // function to copy item content
    var updateItemContent = function (item) {
        var _a, _b, _c;
        var destinationItemOptions = __assign(__assign({}, destinationOptions), { id: item.description.id });
        return parallel.apply(void 0, __spreadArray(__spreadArray([
            // update description
            updateItemDescription(destinationItemOptions, item.description)], (item.data
            ? [
                addItemData(destinationItemOptions, (_b = (_a = item.data) === null || _a === void 0 ? void 0 : _a.text) !== null && _b !== void 0 ? _b : (_c = item.data) === null || _c === void 0 ? void 0 : _c.file),
            ]
            : []), false), (item.resources
            ? [
                parallel.apply(void 0, item.resources.map(function (resource) {
                    return addItemResource(destinationItemOptions, resource);
                })),
            ]
            : []), false));
    };
    // function to copy features to a new feature layer
    var copyLayerFeatures = function (source, destination, maxRecordCount, page) {
        if (maxRecordCount === void 0) { maxRecordCount = 0; }
        if (page === void 0) { page = 0; }
        return series(
        // retrieve feature chunk from source service
        getFeatures(requestOptions, source, {
            outFields: '*',
            resultOffset: page * maxRecordCount,
            resultRecordCount: maxRecordCount,
            returnGeometry: true,
            where: '1=1',
        }), function (_a) {
            var exceededTransferLimit = _a.exceededTransferLimit, features = _a.features;
            return parallel(
            // add features to destination service
            features.length &&
                addFeatures(destinationOptions, destination, features), 
            // copy next chunk if transfer limit exceeded, otherwise return final count
            exceededTransferLimit
                ? copyLayerFeatures(source, destination, features.length, page + 1)
                : Promise.resolve(page * maxRecordCount + features.length)).then(function (_a) {
                var count = _a[1];
                return count;
            });
        });
    };
    var steps = [
        (function () {
            if (options.service) {
                return series(
                // get source item
                getItemContent(requestOptions, options), 
                // get source service definition
                function (item) {
                    return getSourceServiceDefinition(item.description.url).then(function (service) { return ({
                        item: item,
                        service: service,
                    }); });
                }, 
                // create service
                function (source) {
                    return createDestinationService(
                    // name service from item title
                    source.item.description.title, source.service).then(function (destination) { return ({
                        source: source,
                        // merge source item content into destination
                        destination: __assign(__assign({}, destination), { item: __assign(__assign(__assign({}, source.item), destination.item), { description: __assign(__assign({}, source.item.description), destination.item.description) }) }),
                    }); });
                }, 
                // add data & resources to item
                function (_a) {
                    var source = _a.source, destination = _a.destination;
                    // NOTE: layer ids are not preserved on /addToDefinition call
                    //       special handling is needed to accomodate
                    //       destination data may be modified as function side effect
                    var idLookup = handleLayerIdChange(source, destination);
                    return parallel.apply(void 0, __spreadArray([
                        // add content
                        updateItemContent(destination.item).then(function (_a) {
                            var item = _a[0], results = _a.slice(1);
                            return (__assign(__assign({}, item), { resources: results.find(Array.isArray) }));
                        })], (options.service.data
                        ? __spreadArray([], Object.entries(idLookup).map(function (_a) {
                            var sourceId = _a[0], destinationId = _a[1];
                            return copyLayerFeatures("".concat(source.item.description.url, "/").concat(sourceId), "".concat(destination.item.description.url, "/").concat(destinationId));
                        }), true) : []), false)).then(function (_a) {
                        var _b;
                        var info = _a[0];
                        return (__assign(__assign({}, info), { folder: (_b = destinationOptions.folderId) !== null && _b !== void 0 ? _b : '' }));
                    });
                });
            }
            return copyBaseItem(requestOptions, destinationOptions, options);
        })(),
    ];
    return createOperation(WebLayerItemWorkflowOperationTypes.COPY_WEB_LAYER_ITEM, steps, requestOptions);
}
/**
 * Copy a web layer item, optionally cloning feature data.
 *
 * @param requestOptions - Options for the source item request
 * @param destinationOptions - Options for the destination request
 * @param options - Copy options
 * @returns A PProgress Promise that will resolve with the status of the copy.
 */
export default function copyWebLayerItem(requestOptions, destinationOptions, options) {
    var steps = [
        (function () {
            if (options.item.type.name === ItemTypes.FEATURE_SERVICE) {
                return copyFeatureServiceItem(requestOptions, destinationOptions, options);
            }
            return copyBaseItem(requestOptions, destinationOptions, options);
        })(),
    ];
    return createOperation(WebLayerItemWorkflowOperationTypes.COPY_WEB_LAYER_ITEM, steps, requestOptions);
}
//# sourceMappingURL=copy.js.map